package com.blue.penecu;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.UUID;
import android.bluetooth.BluetoothAdapter;
import android.bluetooth.BluetoothDevice;
import android.bluetooth.BluetoothSocket;
import android.content.Context;
import android.os.Handler;

public class BluetoothClientC {
	/****
	 * クラス定数宣言
	 ********/
	public static final int MESSAGE_STATECHANGE = 1;
	public static final int MESSAGE_READ = 2;
	public static final int Max_Size = 4100;
	// SPPのUUIDをセット
	private static final UUID MY_UUID = UUID.fromString("00001101-0000-1000-8000-00805F9B34FB");
	public static final int STATE_NONE = 0;
	public static final int STATE_CONNECTING = 1;
	public static final int STATE_CONNECTED = 2;
	private BluetoothAdapter m_BTadapter;
	private Handler m_handler;
	private int state;
	private ConnectThread connecting;
	static final int m_ntimeout = 3000;        // ソケット接続の待ち時間最大値
	BluetoothSocket m_BTsocket;
	BluetoothDevice m_BTdevice;
//	private ConnectedThread transfer;

	public Handler getHandler() {
		return m_handler;
	}

	// コンストラクタによるAdapterのインスタンス生成
	public BluetoothClientC(Context context, Handler handler) {
		m_BTadapter = BluetoothAdapter.getDefaultAdapter();
		m_handler = handler;
		state = STATE_NONE;
	}

	//----------------------------Bluetoothの接続実行同期化処理
	public synchronized void connect(BluetoothDevice device) {
		m_BTdevice = device;
		if (state == STATE_CONNECTING) {
			if (connecting != null) {                // 既に接続中なら
				connecting.cancel();                // いったんクローズする
				connecting = null;
			}
		}
//		if (transfer != null) {						// 既に接続済みなら
//			transfer.cancel();						// いったんクローズ
//			transfer = null;
//		}
		connecting = new ConnectThread(device);        // 接続スレッドを生成
		connecting.start();                            // スレッド開始
		setState(STATE_CONNECTING);                    // 状態返送
	}

	//----------------------------Bluetoothの再接続
	public synchronized  void reconnect() {
		reconnectT thread = new reconnectT(m_BTdevice, m_BTsocket);
		thread.start();
	}

	//-------------------------- Bluetooth接続処理スレッド

	private class ConnectThread extends Thread {
		private BluetoothDevice BTdevice;
		private BluetoothSocket BTsocket;

		// UUIDでリモート端末との接続用ソケットの生成
		public ConnectThread(BluetoothDevice device) {
			try {
				this.BTdevice = device;
				BTsocket = device.createRfcommSocketToServiceRecord(MY_UUID);
				m_BTsocket = BTsocket;
			} catch (IOException e) {
			}
		}

		// 接続実行処理
		public void run() {
			m_BTadapter.cancelDiscovery();            // 検索処理終了
			try {
				BTsocket.connect();                    // 接続中同期化処理へ
			} catch (IOException e) {
				setState(STATE_NONE);                // 状態返送
				try {
					BTsocket.close();                // エラーならクローズ
				} catch (IOException e2) {
				}
				return;
			}
			synchronized (BluetoothClientC.this) {    // 同期を取る
				connecting = null;                    // ステート初期化
			}
			connected(BTsocket, BTdevice);            // 通信開始同期化処理へ
		}

		// Blutooth接続を切り離す処理
		public void cancel() {
			try {
				BTsocket.close();
			} catch (IOException e) {
			}
		}
	}

	//------------------------------- Timeoutで切断した場合に
	private class reconnectT extends Thread {
		private BluetoothDevice BTdevice;
		private BluetoothSocket BTsocket;

		// UUIDでリモート端末との接続用ソケットの生成
		public reconnectT(BluetoothDevice device, BluetoothSocket sock) {
			BTdevice = device;
			BTsocket = 	sock;
		}

		public void run() {
			try {
				while (true) {
					try {
						BTsocket.close();
						break;
					} catch (IOException ex) {
					}           // エラーならクローズ
				}
				try { Thread.sleep(10); 	} catch (InterruptedException ex) { 	}
				//setState(STATE_CONNECTING);
				while(true) {
					try {
						BTsocket = m_BTdevice.createRfcommSocketToServiceRecord(MY_UUID);
						BTsocket.connect();                    // 接続中同期化処理へ
						break;
					}
					catch (IOException e) {

					}
				}
			} catch (Exception e) {
			}

			connected(BTsocket, BTdevice);            // 通信開始同期化処理へ
		}
	}

	/******
	 * Bluetooth通信開始同期化処理
	 ******/
	public synchronized void connected(BluetoothSocket socket, BluetoothDevice device) {
		if (connecting != null) {                    // 既に接続中なら
			connecting.cancel();                    // いったんクローズ
			connecting = null;
		}

//		if (transfer != null) {						// 既に接続済みなら
//			transfer.cancel();						// いったんクローズ
//			transfer = null;
//		}
//		transfer = new ConnectedThread(socket);		// 送受信スレッドを生成
//		transfer.start();							// スレッド開始

		m_BTsocket = socket;
		m_BTdevice = device;
		setState(STATE_CONNECTED);                    // 状態返送
	}

	public class BTConnectionException extends Exception {
		public BTConnectionException(String strmsg) {
			super(strmsg);
		}
	}

	//--------------------------------- 実際のBluetoothソケットでの通信処理
	class comthread extends Thread
	{
		protected  int  m_result, m_nsize;
		protected  byte[]	m_buff;
		protected  boolean m_isRead;
		InputStream m_input;
		OutputStream m_output;

		public int read(BluetoothSocket sock, byte[] buff, int nread) {
			try {
				m_input = sock.getInputStream();
			}
			catch (IOException e) {
				return -1;
			}
			m_isRead = true;
			m_nsize = nread;
			m_buff = buff;
			this.start();
			try {
				int	i;
				for(i = 0; i < m_ntimeout; i++) {
					if (this.isAlive())
						sleep(1);
					else
						break;
				}
				if (i >= m_ntimeout)
					return -9;

			}
			catch (InterruptedException e) {
				return -2;
			}
			return m_result;
		}

		public int write(BluetoothSocket sock, byte[] buff, int nwrite) {
			try {
				m_output = sock.getOutputStream();
			}
			catch (IOException e) {
				return -1;
			}
			m_isRead = false;
			m_nsize = nwrite;
			m_buff = buff;
			this.start();
			try {
				int	i;
				for(i = 0; i < m_ntimeout; i++) {
					if (this.isAlive())
						sleep(1);
					else
						break;
				}
				if (i >= m_ntimeout)
					return -9;

			}
			catch (InterruptedException e) {
				return -2;
			}
			return m_result;
		}

		public void run() {
			try {
				int ret;
				if(m_isRead) {
					ret = m_input.read(m_buff, 0, m_nsize);
				}
				else {
					m_output.write(m_buff, 0, m_nsize);
					ret = m_nsize;
				}
				synchronized (this)	{
					m_result = ret;
				}
			} catch (IOException e) {
				m_result = -1;
			}
		}
	}

	protected int  m_nread;
	protected int  m_result;
	//------ 受信処理 ---------//
	public int read(byte[] buf){
		return  read(buf, buf.length);
	}
	public  int read(final byte[] buf, final int ncnt){ return read(buf, 0, ncnt); }
	public synchronized int read(final byte[] buf, int offset, final int ncnt){
		int  ret;

		try {
			InputStream input = m_BTsocket.getInputStream();
			int bytes = input.read(buf, offset, ncnt);        // 受信実行
			ret = bytes;
		}
		catch (IOException e) {
			//setState(STATE_NONE);				// 状態返送
			m_handler.obtainMessage(999, e.getMessage()).sendToTarget();
			ret = -1;
		}
		return ret;

/*
		comthread comt = new comthread();
		if((ret = comt.read(m_BTsocket, buf, ncnt)) == -9)
			reconnect();
		return ret;
*/
	}

	//------ 送信処理 ---------//
	public int write(byte c) {
		return write(new byte[] {c}, 1);
	}

	public int write(byte[] buf) {
		return write(buf, buf.length);
	}

	public synchronized int write(byte[] buf, int length) {
		int		ret;

		try {
			OutputStream output = m_BTsocket.getOutputStream();	// 送信出力
			output.write(buf, 0, length);
			ret = 0;
		}
		catch (IOException e) {
			m_handler.obtainMessage(999, e.getMessage()).sendToTarget();
			ret = -1;
		}
		return ret;
/*
		comthread comt = new comthread();
		if((ret =comt.write(m_BTsocket, buf, length)) == -9)
			reconnect();
		return ret;
*/
	}

	public synchronized void cancel() {
		try {
			m_BTsocket.close();
		}
		catch (IOException e) {
		}
	}


	//------ 状態の通知メソッド---------//
	private synchronized void setState(int state) {
		this.state = state;
		m_handler.obtainMessage(MESSAGE_STATECHANGE, state, -1).sendToTarget();
	}
	//------ 状態の取得メソッド ---------//
	public synchronized int getState() {
		return state;
	}
	 //------ Bluetoothの切断メソッド ---------//
	public synchronized void stop() {
		if (null != connecting) {
			connecting.cancel();
			connecting = null;
		}
//		if (null != transfer) {
//			transfer.cancel();
//			transfer = null;
//		}
		this.cancel();
		setState(STATE_NONE);					// 状態返送
	}
}
